@@ -6,15 +6,18 @@
     Tests the form parsing capabilties.  Some of them are also tested from
     the wrappers.
 
-    :copyright: (c) 2010 by the Werkzeug Team, see AUTHORS for more details.
+    :copyright: (c) 2011 by the Werkzeug Team, see AUTHORS for more details.
     :license: BSD, see LICENSE for more details.
 """
 from nose.tools import assert_raises
 from os.path import join, dirname, abspath
 from cStringIO import StringIO
 
-from werkzeug import Client, Request, Response, parse_form_data, \
-     create_environ, FileStorage
+from werkzeug import formparser
+from werkzeug.datastructures import FileStorage
+from werkzeug.formparser import parse_form_data
+from werkzeug.test import Client, create_environ, create_environ
+from werkzeug.wrappers import Request, Response
 from werkzeug.exceptions import RequestEntityTooLarge
 
 
@@ -115,6 +118,15 @@ def test_multipart():
                                content_length=len(data))
         assert response.data == repr(text)
 
+def test_ie7_unc_path():
+    client = Client(form_data_consumer, Response)
+    data_file = join(dirname(__file__), 'multipart', 'ie7_full_path_request.txt')
+    data = get_contents(data_file)
+    boundary = '---------------------------7da36d1b4a0164'
+    response = client.post('/?object=cb_file_upload_multiple', data=data, content_type=
+                               'multipart/form-data; boundary="%s"' % boundary, content_length=len(data))
+    lines = response.data.split('\n', 3)
+    assert lines[0] == repr(u'Sellersburg Town Council Meeting 02-22-2010doc.doc'), lines[0]
 
 def test_end_of_file_multipart():
     """Test for multipart files ending unexpectedly"""
@@ -134,6 +146,26 @@ def test_end_of_file_multipart():
     assert not data.form
 
 
+def test_broken_multipart():
+    """Broken multipart does not break the applicaiton"""
+    data = (
+        '--foo\r\n'
+        'Content-Disposition: form-data; name="test"; filename="test.txt"\r\n'
+        'Content-Transfer-Encoding: base64\r\n'
+        'Content-Type: text/plain\r\n\r\n'
+        'broken base 64'
+        '--foo--'
+    )
+    _, form, files = parse_form_data(create_environ(data=data, method='POST',
+                                     content_type='multipart/form-data; boundary=foo'))
+    assert not files
+    assert not form
+
+    assert_raises(ValueError, parse_form_data, create_environ(data=data, method='POST',
+                  content_type='multipart/form-data; boundary=foo'),
+                  silent=False)
+
+
 def test_multipart_file_no_content_type():
     """Chrome does not always provide a content type."""
     data = (
@@ -180,8 +212,10 @@ def test_multipart_headers():
                               content_type='multipart/form-data; boundary=foo',
                               method='POST')
     foo = req.files['foo']
-    assert foo.content_type == 'text/plain'
-    assert foo.headers['content-type'] == 'text/plain; charset=utf-8'
+    assert foo.mimetype == 'text/plain'
+    assert foo.mimetype_params == {'charset': 'utf-8'}
+    assert foo.headers['content-type'] == foo.content_type \
+        == 'text/plain; charset=utf-8'
     assert foo.headers['x-custom-header'] == 'blah'
 
 
@@ -211,12 +245,6 @@ def test_nonstandard_line_endings():
 def test_limiting():
     """Test the limiting features"""
     data = 'foo=Hello+World&bar=baz'
-    req = Request.from_values(input_stream=StringIO(data),
-                              content_length=len(data),
-                              content_type='application/x-www-form-urlencoded',
-                              method='POST')
-    req.max_content_length = 4
-
     req = Request.from_values(input_stream=StringIO(data),
                               content_length=len(data),
                               content_type='application/x-www-form-urlencoded',
@@ -269,3 +297,60 @@ def test_limiting():
                               method='POST')
     req.max_form_memory_size = 400
     assert req.form['foo'] == 'Hello World'
+
+
+def test_large_file():
+    """Test a largish file."""
+    data = 'x' * (1024 * 600)
+    req = Request.from_values(data={'foo': (StringIO(data), 'test.txt')},
+                              method='POST')
+    # make sure we have a real file here, because we expect to be
+    # on the disk.  > 1024 * 500
+    assert isinstance(req.files['foo'].stream, file)
+
+
+def test_lowlevel():
+    """Lowlevel formparser tests"""
+    assert formparser._line_parse('foo') == ('foo', False)
+    assert formparser._line_parse('foo\r\n') == ('foo', True)
+    assert formparser._line_parse('foo\r') == ('foo', True)
+    assert formparser._line_parse('foo\n') == ('foo', True)
+
+    lineiter = iter('\n\n\nfoo\nbar\nbaz'.splitlines(True))
+    line = formparser._find_terminator(lineiter)
+    assert line == 'foo'
+    assert list(lineiter) == ['bar\n', 'baz']
+    assert formparser._find_terminator([]) == ''
+    assert formparser._find_terminator(['']) == ''
+
+    assert_raises(ValueError, formparser.parse_multipart, None, '', 20)
+    assert_raises(ValueError, formparser.parse_multipart, None, 'broken  ', 20)
+
+    data = '--foo\r\n\r\nHello World\r\n--foo--'
+    assert_raises(ValueError, formparser.parse_multipart, StringIO(data), 'foo', len(data))
+
+    data = '--foo\r\nContent-Disposition: form-field; name=foo\r\n' \
+           'Content-Transfer-Encoding: base64\r\n\r\nHello World\r\n--foo--'
+    assert_raises(ValueError, formparser.parse_multipart, StringIO(data), 'foo', len(data))
+
+    data = '--foo\r\nContent-Disposition: form-field; name=foo\r\n\r\nHello World\r\n'
+    assert_raises(ValueError, formparser.parse_multipart, StringIO(data), 'foo', len(data))
+
+    x = formparser.parse_multipart_headers(['foo: bar\r\n', ' x test\r\n'])
+    assert x['foo'] == 'bar\n x test'
+    assert_raises(ValueError, formparser.parse_multipart_headers, ['foo: bar\r\n', ' x test'])
+
+
+def test_bad_newline_bad_newline_assumption():
+    """Make sure we don't eat up stuff that is not a newline"""
+    class ISORequest(Request):
+        charset = 'latin1'
+    contents = 'U2vlbmUgbORu'
+    data = '--foo\r\nContent-Disposition: form-data; name="test"\r\n' \
+           'Content-Transfer-Encoding: base64\r\n\r\n' + \
+           contents + '\r\n--foo--'
+    req = ISORequest.from_values(input_stream=StringIO(data),
+                                 content_length=len(data),
+                                 content_type='multipart/form-data; boundary=foo',
+                                 method='POST')
+    assert req.form['test'] == u'Sk\xe5ne l\xe4n'
